@@ -27,19 +27,24 @@ for to log, configure and control the run-time of your GUI app via command line
 application events
 ------------------
 
-The following application events are fired exactly one time at startup:
+The following application events are fired exactly one time at startup in the following order:
 
 * `on_app_init`: fired **after** :class:`ConsoleApp` app instance got initialized (detected config files)
   and **before** the image and sound resources and app states get loaded and the GUI framework app class instance gets
   initialized.
 * `on_app_start`: fired **after** the call of the method :meth:`~MainAppBase.run_app` and **before** the command line
   options get parsed.
-* `on_app_run`: fired **after** the parsing of the command line arguments and options and **before** the framework event
+* `on_app_run`: fired **after** the parsing of the command line arguments and options and **before** all portion
+  resources got imported.
+* `on_app_build`: fired **after** all portion resources got loaded/imported and **before** the framework event
   loop of the used GUI framework gets started.
 
+.. note::
+    The `on_app_build` application event has to be fired by the used GUI framework.
+
 .. hint::
     Depending on the used gui framework there can be more app start events. E.g. the :mod:`ae.kivy_app` module fires the
-    events `on_app_build`, `on_app_built` and `on_app_started` (all of them fired after `on_app_run`). See also
+    events `on_app_built` and `on_app_started` (all of them fired after `on_app_run` and `on_app_build`). See also
     :ref:`kivy app class events`.
 
 
@@ -277,19 +282,24 @@ integrate new gui framework
 For to integrate a new Python GUI framework you have to declare a new class that inherits from :class:`MainAppBase` and
 implements at least the abstract method :meth:`~MainAppBase.init_app`.
 
-A minimal implementation of this method would look like::
+Additionally and for to load the resources of the app (after the portions resources got loaded) the event `on_app_build`
+has to be fired, executing the :meth:`MainAppBase.on_app_build` method. This could be done directly from within
+:meth:`~MainAppBase.init_app` or by redirecting one of the events of the app instance of the GUI framework.
+
+A minimal implementation of the :meth:`~MainAppBase.init_app` method would look like the following::
 
     def init_app(self):
+        self.call_method('on_app_build')
         return None, None
 
-Most GUI frameworks are providing classes that need to be instantiated for the application, the root layout of the app
-and for each window. For to keep a reference to these instances within your main app class you can use the attributes
-:attr:`~MainAppBase.framework_app`, :attr:`~MainAppBase.framework_root` and :attr:`~MainAppBase.framework_win` of
-:class:`MainAppBase`.
+Most GUI frameworks are providing classes that need to be instantiated on application startup, like e.g. the
+instance of the GUI framework app class, the root widget or layout of the main GUI framework window(s). For to keep a
+reference to these instances within your main app class you can use the attributes :attr:`~MainAppBase.framework_app`,
+:attr:`~MainAppBase.framework_root` and :attr:`~MainAppBase.framework_win` of the class :class:`MainAppBase`.
 
 The initialization of the attributes :attr:`~MainAppBase.framework_app`, :attr:`~MainAppBase.framework_root` and
 :attr:`~MainAppBase.framework_win` is optional and can be done e.g. within :meth:`~MainAppBase.init_app` or in the later
-called `build` method of the framework app instance.
+called `on_app_build` application event fired by the framework app instance.
 
 .. note::
     If :attr:`~MainAppBase.framework_win` is set to a window instance, then the window instance has to provide a `close`
@@ -304,8 +314,12 @@ A typical implementation of a framework-specific main app class looks like::
             self.framework_app = NewFrameworkAppClass()
             self.framework_win = MainWindowClassOfNewFramework()
 
+            # return callables for to start/stop the event loop of the GUI framework
             return self.framework_app.start, self.framework_app.stop
 
+In this example the `on_app_build` application event gets fired either from within the `start` method of the
+framework app instance or by an event provided by the GUI framework.
+
 :meth:`~MainAppBase.init_app` will be executed only once at the main app class instantiation. Only the main app instance
 has to initialize the GUI framework for to prepare the app startup and has to return at least a callable for to start
 the event loop of the GUI framework.
@@ -314,7 +328,7 @@ the event loop of the GUI framework.
     Although not recommended because of possible namespace conflicts, one could e.g. alternatively integrate the
     framework application class as a mixin to the main app class.
 
-For to finally startup the app the :meth:`~MainAppClass.run_app` method has to be called from the main module of your
+For to initiate the app startup the :meth:`~MainAppClass.run_app` method has to be called from the main module of your
 app project. :meth:`~MainAppBase.run_app` will then start the GUI event loop by calling the first method that got
 returned by :meth:`~MainAppBase.init_app`.
 
@@ -337,24 +351,27 @@ The sound files provides by this portion are taken from:
 """
 import os
 import re
+import shutil
 
 from abc import ABC, abstractmethod
 from copy import deepcopy
 from typing import Any, Callable, Dict, List, Optional, Tuple, Type
 
-from ae.base import NAME_PARTS_SEP, snake_to_camel                                      # type: ignore
+from ae.base import NAME_PARTS_SEP, now_str, snake_to_camel                             # type: ignore
+from ae.paths import Collector, norm_path, path_folders                                 # type: ignore
 from ae.updater import check_all                                                        # type: ignore
 from ae.inspector import stack_var                                                      # type: ignore
 from ae.files import FilesRegister, RegisteredFile                                      # type: ignore
-from ae.i18n import default_language, load_language_texts                               # type: ignore
+from ae.i18n import default_language, load_language_texts, register_translations_path   # type: ignore
 from ae.core import DEBUG_LEVELS                                                        # type: ignore
 from ae.console import ConsoleApp                                                       # type: ignore
 
 
-__version__ = '0.1.54'
+__version__ = '0.1.58'
 
 
 AppStateType = Dict[str, Any]                   #: app state config variable type
+EventKwargsType = Dict[str, Any]                #: change flow event kwargs type
 
 APP_STATE_SECTION_NAME = 'aeAppState'           #: config section name for to store app state
 
@@ -385,14 +402,14 @@ FLOW_ACTION_RE = re.compile("[a-z0-9]+")        #: regular expression detecting
 FLOW_OBJECT_RE = re.compile("[A-Za-z0-9_]+")    #: regular expression detecting invalid characters in flow object string
 
 
-FOUND_IMAGES = FilesRegister()                  #: register of image files found in portions/packages at import time
-FOUND_SOUNDS = FilesRegister()                  #: register of sound files found in portions/packages at import time
+PORTIONS_IMAGES = FilesRegister()               #: register of image files found in portions/packages at import time
+PORTIONS_SOUNDS = FilesRegister()               #: register of sound files found in portions/packages at import time
 
 
 check_all()     # let ae_updater module check/install any outstanding updates or new app versions
 
 
-def ensure_tap_kwargs_refs(init_kwargs: Dict[str, Any], tap_widget: Any):
+def ensure_tap_kwargs_refs(init_kwargs: EventKwargsType, tap_widget: Any):
     """ ensure that the passed widget.__init__ kwargs dict contains a reference to itself within kwargs['tap_kwargs'].
 
     :param init_kwargs:         kwargs of the widgets __init__ method.
@@ -515,32 +532,38 @@ def id_of_flow(action: str, obj: str = "", key: str = "") -> str:
     return cid
 
 
-def register_package_images(*skip_modules: str):              # pragma: no cover
+def register_package_images():
     """ call from module scope of the package for to register/add image/img resources path.
 
-    :param skip_modules:        list of packages/modules/portions to skip (pass at least `'ae.inspector'` for to
-                                disable the default skip modules, e.g. if you want to register resources provided by a
-                                ae namespace portion that gets by default skipped).
-        """
-    global FOUND_IMAGES
+    No parameters needed because we use here :func:`~ae.inspector.stack_var` helper function for to determine the
+    the module file path via the `__file__` module variable of the caller module in the call stack. In this call
+    we have to overwrite the default value (:data:`~ae.inspector.SKIPPED_MODULES`) of the
+    :paramref:`~ae.inspector.stack_var.skip_modules` parameter for to not skip ae portions that are providing
+    package resources and are listed in the :data:`~ae.inspector.SKIPPED_MODULES`, like e.g. :mod:`ae.gui_app` and
+    :mod:`ae.gui_help` (passing empty string '' for to overwrite default skip list).
+    """
+    global PORTIONS_IMAGES
 
-    package_path = os.path.abspath(os.path.dirname(stack_var('__file__', *skip_modules)))
+    package_path = os.path.abspath(os.path.dirname(stack_var('__file__', '')))
     search_path = os.path.join(package_path, 'img/**')
-    FOUND_IMAGES.add_paths(search_path)
+    PORTIONS_IMAGES.add_paths(search_path)
 
 
-def register_package_sounds(*skip_modules: str):              # pragma: no cover
+def register_package_sounds():
     """ call from module scope of the package for to register/add sound file resources.
 
-    :param skip_modules:        list of packages/modules/portions to skip (pass at least `'ae.inspector'` for to
-                                disable the default skip modules, e.g. if you want to register resources provided by a
-                                ae namespace portion that gets by default skipped).
+    No parameters needed because we use here :func:`~ae.inspector.stack_var` helper function for to determine the
+    the module file path via the `__file__` module variable of the caller module in the call stack. In this call
+    we have to overwrite the default value (:data:`~ae.inspector.SKIPPED_MODULES`) of the
+    :paramref:`~ae.inspector.stack_var.skip_modules` parameter for to not skip ae portions that are providing
+    package resources and are listed in the :data:`~ae.inspector.SKIPPED_MODULES`, like e.g. :mod:`ae.gui_app`
+    :mod:`ae.gui_help` (passing empty string '' for to overwrite default skip list).
     """
-    global FOUND_SOUNDS
+    global PORTIONS_SOUNDS
 
-    package_path = os.path.abspath(os.path.dirname(stack_var('__file__', *skip_modules)))
+    package_path = os.path.abspath(os.path.dirname(stack_var('__file__', '')))
     search_path = os.path.join(package_path, 'snd/**')
-    FOUND_SOUNDS.add_paths(search_path)
+    PORTIONS_SOUNDS.add_paths(search_path)
 
 
 def replace_flow_action(flow_id: str, new_action: str):
@@ -553,7 +576,7 @@ def replace_flow_action(flow_id: str, new_action: str):
     return id_of_flow(new_action, *flow_key_split(flow_action_split(flow_id)[1]))
 
 
-def update_tap_kwargs(widget: Any, popup_kwargs: Optional[Dict[str, Any]] = None, **tap_kwargs) -> Dict[str, Any]:
+def update_tap_kwargs(widget: Any, popup_kwargs: Optional[EventKwargsType] = None, **tap_kwargs) -> EventKwargsType:
     """ update widget's tap_kwargs property and return the updated dictionary (for kv rule of tap_kwargs).
 
     :param widget:              widget with tap_kwargs property to be updated.
@@ -610,9 +633,6 @@ class MainAppBase(ConsoleApp, ABC):
 
         self.call_method('on_app_init')
 
-        self.load_images()
-        self.load_sounds()
-
         self._start_event_loop, self._stop_event_loop = self.init_app()
 
         self.load_app_states()
@@ -635,6 +655,29 @@ class MainAppBase(ConsoleApp, ABC):
         """
         return self.cfg_section_variable_names(APP_STATE_SECTION_NAME)
 
+    @staticmethod
+    def backup_config_resources() -> str:   # pragma: no cover
+        """ backup config files and image/sound/translations resources to {ado}<now_str>.
+
+        config files are collected from from {app}, {usr} or {cwd} (the first found file name only).
+        resources are copied from {ado} or {cwd} (only the first found resources root path).
+        """
+        backup_root = norm_path("{ado}") + now_str(sep="_")
+        os.makedirs(backup_root)
+
+        coll = Collector()
+        coll.collect("{app}", "{usr}", "{cwd}", append=("*.ini", "*.cfg"))
+        for file in coll.files:
+            if not os.path.exists(os.path.basename(file)):  # don't overwrite the first one found (the active one)
+                shutil.copy2(file, backup_root)
+
+        coll = Collector(path_scanner=path_folders)
+        coll.collect("{ado}", "{cwd}", append=('img', 'loc', 'snd'), only_first_of='prefix')
+        for path in coll.paths:
+            shutil.copytree(path, os.path.join(backup_root, os.path.basename(path)))
+
+        return backup_root
+
     def change_app_state(self, state_name: str, state_value: Any, send_event: bool = True):
         """ change app state to :paramref:`~change_app_state.state_value` in self.<state_name> and app_state dict item.
 
@@ -671,31 +714,45 @@ class MainAppBase(ConsoleApp, ABC):
 
         self.setup_app_states(app_states)
 
+    # application events - declared in the order they get executed/fired
+
     def on_app_init(self):
         """ default/fallback flow change confirmation event handler. """
-        self.vpo("MainAppBase.on_app_init default/fallback flow change confirmation event handler called")
+        self.vpo("MainAppBase.on_app_init default/fallback event handler called")
 
     def on_app_start(self):
         """ default/fallback flow change confirmation event handler. """
-        self.vpo("MainAppBase.on_app_start default/fallback flow change confirmation event handler called")
+        self.vpo("MainAppBase.on_app_start default/fallback event handler called")
 
     def on_app_run(self):
         """ default/fallback flow change confirmation event handler. """
-        self.vpo("MainAppBase.on_app_run default/fallback flow change confirmation event handler called")
+        self.vpo("MainAppBase.on_app_run default/fallback event handler called")
+
+    def on_app_build(self):
+        """ default/fallback flow change confirmation event handler. """
+        self.vpo("MainAppBase.on_app_build default/fallback event handler called - loading app resources")
+
+        self.load_images()
+
+        self.load_sounds()
+
+        register_translations_path()
+        register_translations_path("{ado}")
+        self.load_translations(self.lang_code)
 
     def on_app_stop(self):
         """ default/fallback flow change confirmation event handler. """
-        self.vpo("MainAppBase.on_app_stop default/fallback flow change confirmation event handler called")
+        self.vpo("MainAppBase.on_app_stop default/fallback event handler called")
 
     def on_app_exit(self):
         """ default/fallback flow change confirmation event handler. """
-        self.vpo("MainAppBase.on_app_exit default/fallback flow change confirmation event handler called")
+        self.vpo("MainAppBase.on_app_exit default/fallback event handler called")
 
     def on_app_quit(self):
         """ default/fallback flow change confirmation event handler. """
-        self.vpo("MainAppBase.on_app_quit default/fallback flow change confirmation event handler called")
+        self.vpo("MainAppBase.on_app_quit default/fallback event handler called")
 
-    def on_debug_level_change(self, level_name: str, _event_kwargs: Dict[str, Any]) -> bool:
+    def on_debug_level_change(self, level_name: str, _event_kwargs: EventKwargsType) -> bool:
         """ debug level app state change flow change confirmation event handler.
 
         :param level_name:      the new debug level name to be set (passed as flow key).
@@ -707,7 +764,7 @@ class MainAppBase(ConsoleApp, ABC):
         self.set_opt('debug_level', debug_level)
         return True
 
-    def on_lang_code_change(self, lang_code: str, _event_kwargs: Dict[str, Any]) -> bool:
+    def on_lang_code_change(self, lang_code: str, _event_kwargs: EventKwargsType) -> bool:
         """ language app state change flow change confirmation event handler.
 
         :param lang_code:       the new language code to be set (passed as flow key). Empty on first app run/start.
@@ -718,7 +775,7 @@ class MainAppBase(ConsoleApp, ABC):
         self.load_translations(lang_code)
         return True
 
-    def on_light_theme_change(self, _flow_key: str, event_kwargs: Dict[str, Any]) -> bool:
+    def on_light_theme_change(self, _flow_key: str, event_kwargs: EventKwargsType) -> bool:
         """ app theme app state change flow change confirmation event handler.
 
         :param _flow_key:       flow key.
@@ -966,7 +1023,7 @@ class MainAppBase(ConsoleApp, ABC):
             deep -= 1
         return flow_path[:deep]
 
-    def on_flow_change(self, flow_id: str, event_kwargs: Dict[str, Any]) -> bool:
+    def on_flow_change(self, flow_id: str, event_kwargs: EventKwargsType) -> bool:
         """ checking if exists a Popup class for the new flow and if yes then open it.
 
         :param flow_id:         new flow id.
@@ -988,7 +1045,7 @@ class MainAppBase(ConsoleApp, ABC):
         return False
 
     @staticmethod
-    def on_flow_popup_close(_flow_key: str, _popup_args: Dict[str, Any]) -> bool:
+    def on_flow_popup_close(_flow_key: str, _popup_args: EventKwargsType) -> bool:
         """ default popup close handler of FlowPopup widget, ensuring update of :attr:`flow_path`.
 
         :param _flow_key:       unused flow key.
@@ -1075,15 +1132,17 @@ class MainAppBase(ConsoleApp, ABC):
     def load_images(self):
         """ load images from app folder img. """
         file_reg = FilesRegister()
+        file_reg.add_register(PORTIONS_IMAGES)
         file_reg.add_paths('img/**')
-        file_reg.add_register(FOUND_IMAGES)
+        file_reg.add_paths('{ado}/img/**')
         self.image_files = file_reg
 
     def load_sounds(self):
         """ load audio sounds from app folder snd. """
         file_reg = FilesRegister()
+        file_reg.add_register(PORTIONS_SOUNDS)
         file_reg.add_paths('snd/**')
-        file_reg.add_register(FOUND_SOUNDS)
+        file_reg.add_paths('{ado}/snd/**')
         self.sound_files = file_reg
 
     def load_translations(self, lang_code: str):
@@ -1215,8 +1274,8 @@ class MainAppBase(ConsoleApp, ABC):
         self.call_method('on_win_pos_size')
 
 
-register_package_images('ae.inspector')   # register base image files of this portion
-register_package_sounds('ae.inspector')   # register base sound files of this portion
+register_package_images()   # register base image files of this portion
+register_package_sounds()   # register base sound files of this portion
 
 module_globals = globals()
 #: used. e.g. by :mod:`ae.gui_help` for execution/evaluation of dynamic code, expressions and f-strings
